@@ -162,9 +162,8 @@ def common_options(f: Callable) -> Callable:
         ),
     )(f)
     f = click.option(
-        "-n",
-        "--nocolor",
-        is_flag=True,
+        "-n/ ",
+        "--nocolor/--color",
         default=None,
         help="No color - output will be without ANSI color codes.",
     )(f)
@@ -366,6 +365,11 @@ def lint_options(f: Callable) -> Callable:
         default=False,
         help="Warn about unneeded '-- noqa:' comments.",
     )(f)
+    f = click.option(
+        "--disregard-sqlfluffignores",
+        is_flag=True,
+        help="Perform the operation regardless of .sqlfluffignore configurations",
+    )(f)
     return f
 
 
@@ -573,11 +577,6 @@ def dump_file_payload(filename: Optional[str], payload: str) -> None:
         "found. This is potentially useful during rollout."
     ),
 )
-@click.option(
-    "--disregard-sqlfluffignores",
-    is_flag=True,
-    help="Perform the operation regardless of .sqlfluffignore configurations",
-)
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def lint(
     paths: tuple[str],
@@ -643,8 +642,12 @@ def lint(
         # add stdin if specified via lone '-'
         if ("-",) == paths:
             if stdin_filename:
-                lnt.config = lnt.config.make_child_from_path(stdin_filename)
-            result = lnt.lint_string_wrapped(sys.stdin.read(), fname="stdin")
+                lnt.config = lnt.config.make_child_from_path(
+                    stdin_filename, require_dialect=False
+                )
+            result = lnt.lint_string_wrapped(
+                sys.stdin.read(), fname="stdin", stdin_filename=stdin_filename
+            )
         else:
             result = lnt.lint_paths(
                 paths,
@@ -835,13 +838,18 @@ def _handle_unparsable(
 
 
 def _stdin_fix(
-    linter: Linter, formatter: OutputStreamFormatter, fix_even_unparsable: bool
+    linter: Linter,
+    formatter: OutputStreamFormatter,
+    fix_even_unparsable: bool,
+    stdin_filename: Optional[str] = None,
 ) -> None:
     """Handle fixing from stdin."""
     exit_code = EXIT_SUCCESS
     stdin = sys.stdin.read()
 
-    result = linter.lint_string_wrapped(stdin, fname="stdin", fix=True)
+    result = linter.lint_string_wrapped(
+        stdin, fname="stdin", fix=True, stdin_filename=stdin_filename
+    )
     templater_error = result.num_violations(types=SQLTemplaterError) > 0
     unfixable_error = result.num_violations(types=SQLLintError, fixable=False) > 0
 
@@ -889,6 +897,7 @@ def _paths_fix(
     show_lint_violations,
     check: bool = False,
     persist_timing: Optional[str] = None,
+    ignore_files: bool = True,
 ) -> None:
     """Handle fixing from paths."""
     # Lint the paths (not with the fix argument at this stage), outputting as we go.
@@ -901,6 +910,7 @@ def _paths_fix(
             paths,
             fix=True,
             ignore_non_existent_files=False,
+            ignore_files=ignore_files,
             processes=processes,
             # If --check is set, then don't apply any fixes until the end.
             apply_fixes=not check,
@@ -1056,6 +1066,7 @@ def _paths_fix(
 def fix(
     force: bool,
     paths: tuple[str],
+    disregard_sqlfluffignores: bool,
     check: bool = False,
     bench: bool = False,
     quiet: bool = False,
@@ -1125,8 +1136,10 @@ def fix(
         # handle stdin case. should output formatted sql to stdout and nothing else.
         if fixing_stdin:
             if stdin_filename:
-                lnt.config = lnt.config.make_child_from_path(stdin_filename)
-            _stdin_fix(lnt, formatter, fix_even_unparsable)
+                lnt.config = lnt.config.make_child_from_path(
+                    stdin_filename, require_dialect=False
+                )
+            _stdin_fix(lnt, formatter, fix_even_unparsable, stdin_filename)
         else:
             _paths_fix(
                 lnt,
@@ -1139,6 +1152,7 @@ def fix(
                 show_lint_violations,
                 check=check,
                 persist_timing=persist_timing,
+                ignore_files=not disregard_sqlfluffignores,
             )
 
 
@@ -1155,6 +1169,7 @@ def fix(
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def cli_format(
     paths: tuple[str],
+    disregard_sqlfluffignores: bool,
     bench: bool = False,
     fixed_suffix: str = "",
     logger: Optional[logging.Logger] = None,
@@ -1227,8 +1242,12 @@ def cli_format(
         # handle stdin case. should output formatted sql to stdout and nothing else.
         if fixing_stdin:
             if stdin_filename:
-                lnt.config = lnt.config.make_child_from_path(stdin_filename)
-            _stdin_fix(lnt, formatter, fix_even_unparsable=False)
+                lnt.config = lnt.config.make_child_from_path(
+                    stdin_filename, require_dialect=False
+                )
+            _stdin_fix(
+                lnt, formatter, fix_even_unparsable=False, stdin_filename=stdin_filename
+            )
         else:
             _paths_fix(
                 lnt,
@@ -1240,6 +1259,7 @@ def cli_format(
                 bench=bench,
                 show_lint_violations=False,
                 persist_timing=persist_timing,
+                ignore_files=not disregard_sqlfluffignores,
             )
 
 
@@ -1360,7 +1380,9 @@ def parse(
         if "-" == path:
             file_config = lnt.config
             if stdin_filename:
-                file_config = file_config.make_child_from_path(stdin_filename)
+                file_config = file_config.make_child_from_path(
+                    stdin_filename, require_dialect=False
+                )
             parsed_strings = [
                 lnt.parse_string(
                     sys.stdin.read(),
