@@ -1,11 +1,13 @@
 """Linter class and helper classes."""
 
+from sqlfluff.core.formatter import FormatterInterface
 from sqlfluff.core.linter.common import ParsedString, RenderedFile, RuleTuple
 from sqlfluff.core.linter.linted_file import LintedFile
 from sqlfluff.core.linter.linter import Linter
 from sqlfluff.core.linter.linting_result import LintingResult
 
 __all__ = (
+    "FormatterInterface",
     "RuleTuple",
     "ParsedString",
     "LintedFile",
