@@ -82,11 +82,14 @@ oracle_dialect.sets("reserved_keywords").update(
         "DELETE",
         "DELETING",
         "DESC",
+        "DISABLE",
         "DISTINCT",
         "DROP",
         "ELSE",
+        "ENABLE",
         "EXCLUSIVE",
         "EXISTS",
+        "EXECUTE",
         "FILE",
         "FLOAT",
         "FOR",
@@ -107,10 +110,12 @@ oracle_dialect.sets("reserved_keywords").update(
         "INTEGER",
         "INTERSECT",
         "INTO",
+        "INVISIBLE",
         "IS",
         "LEVEL",
         "LIKE",
         "LOCK",
+        "LOGGING",
         "LONG",
         "LOOP",
         "MAXEXTENTS",
@@ -118,9 +123,13 @@ oracle_dialect.sets("reserved_keywords").update(
         "MLSLABEL",
         "MODE",
         "MODIFY",
+        "MONITORING",
         "NESTED_TABLE_ID",
         "NOAUDIT",
         "NOCOMPRESS",
+        "NOLOGGING",
+        "NOMONITORING",
+        "NOREVERSE",
         "NOT",
         "NOWAIT",
         "NULL",
@@ -133,6 +142,7 @@ oracle_dialect.sets("reserved_keywords").update(
         "OR",
         "ORDER",
         "OVERFLOW",
+        "PARAMETERS",
         "PCTFREE",
         "PIVOT",
         "PRIOR",
@@ -140,9 +150,11 @@ oracle_dialect.sets("reserved_keywords").update(
         "PROMPT",
         "PUBLIC",
         "RAW",
+        "REBUILD",
         "RENAME",
         "RESOURCE",
         "REVOKE",
+        "REVERSE",
         "ROW",
         "ROWID",
         "ROWNUM",
@@ -166,6 +178,7 @@ oracle_dialect.sets("reserved_keywords").update(
         "UNION",
         "UNIQUE",
         "UNPIVOT",
+        "UNUSABLE",
         "UPDATE",
         "UPDATING",
         "USER",
@@ -174,6 +187,7 @@ oracle_dialect.sets("reserved_keywords").update(
         "VARCHAR",
         "VARCHAR2",
         "VIEW",
+        "VISIBLE",
         "WHEN",
         "WHENEVER",
         "WHERE",
@@ -183,12 +197,15 @@ oracle_dialect.sets("reserved_keywords").update(
 
 oracle_dialect.sets("unreserved_keywords").update(
     [
+        "ABSENT",
         "ACCESSIBLE",
         "AUTHID",
         "BODY",
+        "BULK",
         "BULK_EXCEPTIONS",
         "BULK_ROWCOUNT",
         "BYTE",
+        "COLLECT",
         "COMPILE",
         "COMPOUND",
         "CONSTANT",
@@ -213,6 +230,7 @@ oracle_dialect.sets("unreserved_keywords").update(
         "LOOP",
         "MUTABLE",
         "NESTED",
+        "NEXTVAL",
         "NOCOPY",
         "NOMAXVALUE",
         "NOMINVALUE",
@@ -225,12 +243,14 @@ oracle_dialect.sets("unreserved_keywords").update(
         "PARENT",
         "PERSISTABLE",
         "PIPELINED",
+        "POLYMORPHIC",
         "PRAGMA",
         "PRECEDES",
         "PROFILE",
         "QUOTA",
         "RAISE",
         "RECORD",
+        "RELIES_ON",
         "RESULT_CACHE",
         "RETURNING",
         "REUSE",
@@ -248,6 +268,7 @@ oracle_dialect.sets("unreserved_keywords").update(
 oracle_dialect.sets("bare_functions").clear()
 oracle_dialect.sets("bare_functions").update(
     [
+        "column_value",
         "current_date",
         "current_timestamp",
         "dbtimezone",
@@ -261,7 +282,7 @@ oracle_dialect.sets("bare_functions").update(
 
 oracle_dialect.patch_lexer_matchers(
     [
-        RegexLexer("word", r"[a-zA-Z][0-9a-zA-Z_$#]*", WordSegment),
+        RegexLexer("word", r"[\p{L}][\p{L}\p{N}_$#]*", WordSegment),
         RegexLexer(
             "single_quote",
             r"'([^'\\]|\\|\\.|'')*'",
@@ -304,13 +325,32 @@ oracle_dialect.insert_lexer_matchers(
     # JSON Operators: https://www.postgresql.org/docs/9.5/functions-json.html
     [
         StringLexer("right_arrow", "=>", CodeSegment),
+        StringLexer("assignment_operator", ":=", CodeSegment),
     ],
     before="equals",
 )
 
+oracle_dialect.insert_lexer_matchers(
+    [
+        StringLexer("power_operator", "**", CodeSegment),
+    ],
+    before="star",
+)
+
 oracle_dialect.add(
+    SequenceNextValGrammar=Sequence(
+        Ref("NakedIdentifierSegment"),
+        Ref("DotSegment"),
+        "NEXTVAL",
+        allow_gaps=False,
+    ),
     AtSignSegment=StringParser("@", SymbolSegment, type="at_sign"),
     RightArrowSegment=StringParser("=>", SymbolSegment, type="right_arrow"),
+    AssignmentOperatorSegment=StringParser(
+        ":=", SymbolSegment, type="assignment_operator"
+    ),
+    PowerOperatorSegment=StringParser("**", SymbolSegment, type="binary_operator"),
+    ModOperatorSegment=StringParser("MOD", WordSegment, type="binary_operator"),
     OnCommitGrammar=Sequence(
         "ON",
         "COMMIT",
@@ -612,9 +652,63 @@ oracle_dialect.add(
         Sequence("UPDATING", Bracketed(Ref("QuotedLiteralSegment"), optional=True)),
         "DELETING",
     ),
+    JSONObjectContentSegment=Sequence(
+        OneOf(Ref("StarSegment"), Delimited(Ref("JSONEntrySegment")), optional=True),
+        Ref("JSONOnNullClause", optional=True),
+        Ref("JSONReturningClause", optional=True),
+        Ref.keyword("STRICT", optional=True),
+        Sequence("WITH", "UNIQUE", "KEYS", optional=True),
+    ),
+    JSONEntrySegment=OneOf(
+        Sequence(
+            Ref("JSONRegularEntrySegment"),
+            Sequence("FORMAT", "JSON", optional=True),
+        ),
+        Ref("WildcardIdentifierSegment"),
+    ),
+    JSONRegularEntrySegment=Sequence(
+        OneOf(
+            Sequence(
+                Ref.keyword("KEY", optional=True),
+                Ref("QuotedLiteralSegment"),
+                "VALUE",
+                Ref("ExpressionSegment"),
+            ),
+            Sequence(
+                Ref("ExpressionSegment"),
+                Sequence(Ref("ColonSegment"), Ref("ExpressionSegment"), optional=True),
+            ),
+            Ref("ColumnReferenceSegment"),
+        )
+    ),
+    JSONOnNullClause=Sequence(OneOf("NULL", "ABSENT"), "ON", "NULL"),
+    JSONReturningClause=Sequence(
+        "RETURNING",
+        OneOf(
+            Sequence(
+                "VARCHAR",
+                Bracketed(
+                    Sequence(
+                        Ref("NumericLiteralSegment"),
+                        OneOf("BYTE", "CHAR", optional=True),
+                    ),
+                    optional=True,
+                ),
+                Sequence("WITH", "TYPENAME", optional=True),
+            ),
+            Sequence(
+                OneOf("CLOB", "BLOB"), Ref("SingleIdentifierGrammar", optional=True)
+            ),
+            "JSON",
+        ),
+    ),
 )
 
 oracle_dialect.replace(
+    ColumnConstraintDefaultGrammar=OneOf(
+        ansi_dialect.get_grammar("ColumnConstraintDefaultGrammar"),
+        Ref("SequenceNextValGrammar"),
+    ),
     # https://docs.oracle.com/en/database/oracle/oracle-database/19/sqlrf/DROP-TABLE.html
     DropBehaviorGrammar=Sequence(
         Sequence(
@@ -627,7 +721,7 @@ oracle_dialect.replace(
     ),
     NakedIdentifierSegment=SegmentGenerator(
         lambda dialect: RegexParser(
-            r"[A-Z0-9_]*[A-Z][A-Z0-9_#$]*",
+            r"[\p{L}\p{N}_]*[\p{L}][\p{L}\p{N}_#$]*",
             IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
@@ -644,9 +738,7 @@ oracle_dialect.replace(
         Ref("NamedArgumentSegment"),
     ),
     FunctionContentsGrammar=ansi_dialect.get_grammar("FunctionContentsGrammar").copy(
-        insert=[
-            Ref("ListaggOverflowClauseSegment"),
-        ]
+        insert=[Ref("ListaggOverflowClauseSegment"), Ref("JSONObjectContentSegment")]
     ),
     TemporaryGrammar=Sequence(
         OneOf("GLOBAL", "PRIVATE"),
@@ -768,9 +860,7 @@ oracle_dialect.replace(
                 Ref.keyword("IN", optional=True),
                 OneOf(Ref("DatatypeSegment"), Ref("ColumnTypeReferenceSegment")),
                 Sequence(
-                    OneOf(
-                        Sequence(Ref("ColonSegment"), Ref("EqualsSegment")), "DEFAULT"
-                    ),
+                    OneOf(Ref("AssignmentOperatorSegment"), "DEFAULT"),
                     Ref("ExpressionSegment"),
                     optional=True,
                 ),
@@ -786,7 +876,16 @@ oracle_dialect.replace(
     DelimiterGrammar=Sequence(
         Ref("SemicolonSegment"), Ref("SlashStatementTerminatorSegment", optional=True)
     ),
+    ArithmeticBinaryOperatorGrammar=ansi_dialect.get_grammar(
+        "ArithmeticBinaryOperatorGrammar"
+    ).copy(
+        insert=[
+            Ref("ModOperatorSegment"),
+            Ref("PowerOperatorSegment"),
+        ]
+    ),
     SelectClauseTerminatorGrammar=OneOf(
+        "BULK",
         "INTO",
         "FROM",
         "WHERE",
@@ -799,6 +898,43 @@ oracle_dialect.replace(
 )
 
 
+class AlterIndexStatementSegment(BaseSegment):
+    """An `ALTER INDEX` statement.
+
+    https://docs.oracle.com/en/database/oracle/oracle-database/21/sqlrf/ALTER-INDEX.html
+    If possible, please keep the order below the same as Oracle's doc:
+    """
+
+    type = "alter_index_statement"
+    match_grammar: Matchable = Sequence(
+        "ALTER",
+        "INDEX",
+        Ref("IndexReferenceSegment"),
+        OneOf(
+            Sequence(
+                "REBUILD",
+                OneOf(
+                    "REVERSE",
+                    "NOREVERSE",
+                    optional=True,
+                ),
+            ),
+            Sequence("MONITORING", "USAGE"),
+            Sequence("NOMONITORING", "USAGE"),
+            Sequence("PARAMETERS", Bracketed(Ref("QuotedLiteralSegment"))),
+            Sequence("RENAME", "TO", Ref("IndexReferenceSegment")),
+            "COMPILE",
+            "LOGGING",
+            "NOLOGGING",
+            "ENABLE",
+            "DISABLE",
+            "UNUSABLE",
+            "INVISIBLE",
+            "VISIBLE",
+        ),
+    )
+
+
 class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
     """An `ALTER TABLE` statement.
 
@@ -917,7 +1053,6 @@ class AlterTableConstraintClauses(BaseSegment):
             Ref("TableConstraintSegment"),
         ),
         # @TODO MODIFY
-        # @TODO RENAME
         # @TODO DROP
         # drop_constraint_clause
         Sequence(
@@ -944,6 +1079,13 @@ class AlterTableConstraintClauses(BaseSegment):
             ),
             Ref.keyword("ONLINE", optional=True),
         ),
+        Sequence(
+            "RENAME",
+            "CONSTRAINT",
+            Ref("ObjectReferenceSegment"),
+            "TO",
+            Ref("ObjectReferenceSegment"),
+        ),
     )
 
 
@@ -1002,6 +1144,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("AssignmentStatementSegment"),
             Ref("RecordTypeDefinitionSegment"),
             Ref("DeclareCursorVariableSegment"),
+            Ref("ExecuteImmediateSegment"),
             Ref("FunctionSegment"),
             Ref("IfExpressionStatement"),
             Ref("CaseExpressionSegment"),
@@ -1018,6 +1161,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("ContinueStatementSegment"),
             Ref("RaiseStatementSegment"),
             Ref("ReturnStatementSegment"),
+            Ref("AlterIndexStatementSegment"),
         ],
     )
 
@@ -1368,7 +1512,11 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
         ],
     ).copy(
         insert=[
-            Ref("IntoClauseSegment", optional=True),
+            OneOf(
+                Ref("IntoClauseSegment"),
+                Ref("BulkCollectIntoClauseSegment"),
+                optional=True,
+            ),
         ],
         before=Ref("FromClauseSegment", optional=True),
     )
@@ -1653,6 +1801,7 @@ class DeclareSegment(BaseSegment):
 
     match_grammar = Sequence(
         Ref.keyword("DECLARE", optional=True),
+        Indent,
         AnyNumberOf(
             Delimited(
                 OneOf(
@@ -1678,7 +1827,7 @@ class DeclareSegment(BaseSegment):
                         Sequence("NOT", "NULL", optional=True),
                         Sequence(
                             OneOf(
-                                Sequence(Ref("ColonSegment"), Ref("EqualsSegment")),
+                                Ref("AssignmentOperatorSegment"),
                                 "DEFAULT",
                             ),
                             Ref("ExpressionSegment"),
@@ -1692,8 +1841,10 @@ class DeclareSegment(BaseSegment):
                 ),
                 delimiter=Ref("DelimiterGrammar"),
                 terminators=["BEGIN", "END"],
-            )
+            ),
+            min_times=1,
         ),
+        Dedent,
     )
 
 
@@ -1768,7 +1919,7 @@ class RecordTypeDefinitionSegment(BaseSegment):
                     Sequence(
                         Sequence("NOT", "NULL", optional=True),
                         OneOf(
-                            Sequence(Ref("ColonSegment"), Ref("EqualsSegment")),
+                            Ref("AssignmentOperatorSegment"),
                             "DEFAULT",
                         ),
                         Ref("ExpressionSegment"),
@@ -1827,11 +1978,51 @@ class DeclareCursorVariableSegment(BaseSegment):
             ),
             optional=True,
         ),
-        Sequence("IS", Ref("SelectStatementSegment"), optional=True),
+        Sequence("IS", Indent, Ref("SelectStatementSegment"), Dedent, optional=True),
         Ref("DelimiterGrammar", optional=True),
     )
 
 
+class ExecuteImmediateSegment(BaseSegment):
+    """An `EXECUTE IMMEDIATE` statement.
+
+    https://docs.oracle.com/en/database/oracle/oracle-database/23/lnpls/EXECUTE-IMMEDIATE-statement.html
+    """
+
+    type = "execute_immediate_statement"
+
+    match_grammar = Sequence(
+        "EXECUTE",
+        "IMMEDIATE",
+        Indent,
+        Ref("ExpressionSegment"),
+        OneOf(
+            Ref("IntoClauseSegment"),
+            Ref("BulkCollectIntoClauseSegment"),
+            optional=True,
+        ),
+        Sequence(
+            "USING",
+            Delimited(
+                Sequence(
+                    OneOf("IN", "OUT", Sequence("IN", "OUT"), optional=True),
+                    Ref("ExpressionSegment"),
+                ),
+            ),
+            optional=True,
+        ),
+        Sequence(
+            OneOf("RETURNING", "RETURN"),
+            OneOf(
+                Ref("IntoClauseSegment"),
+                Ref("BulkCollectIntoClauseSegment"),
+            ),
+            optional=True,
+        ),
+        Dedent,
+    )
+
+
 class BeginEndSegment(BaseSegment):
     """A `BEGIN/END` block.
 
@@ -1840,6 +2031,24 @@ class BeginEndSegment(BaseSegment):
     https://docs.oracle.com/en/database/oracle/oracle-database/23/lnpls/block.html
     """
 
+    _when_clause = Sequence(
+        "WHEN",
+        OneOf(
+            "OTHERS",
+            Sequence(
+                Ref("SingleIdentifierGrammar"),
+                AnyNumberOf(
+                    Sequence(
+                        "OR",
+                        Ref("SingleIdentifierGrammar"),
+                    )
+                ),
+            ),
+        ),
+        "THEN",
+        Ref("OneOrMoreStatementsGrammar"),
+    )
+
     type = "begin_end_block"
     match_grammar = Sequence(
         Ref("DeclareSegment", optional=True),
@@ -1848,21 +2057,10 @@ class BeginEndSegment(BaseSegment):
         Ref("OneOrMoreStatementsGrammar"),
         Sequence(
             "EXCEPTION",
-            "WHEN",
-            OneOf(
-                "OTHERS",
-                Sequence(
-                    Ref("SingleIdentifierGrammar"),
-                    AnyNumberOf(
-                        Sequence(
-                            "OR",
-                            Ref("SingleIdentifierGrammar"),
-                        )
-                    ),
-                ),
-            ),
-            "THEN",
-            Ref("OneOrMoreStatementsGrammar"),
+            # Using AnyNumberOf with min_times=1 is not greedy enough to grab multiple
+            # exceptions here. So define it once, then have AnyNumberOf after.
+            _when_clause,
+            AnyNumberOf(_when_clause),
             optional=True,
         ),
         Dedent,
@@ -2001,7 +2199,9 @@ class CreateTypeBodyStatementSegment(BaseSegment):
         Ref("TypeReferenceSegment"),
         Ref("SharingClauseGrammar", optional=True),
         OneOf("IS", "AS"),
+        Indent,
         Ref("ElementSpecificationGrammar"),
+        Dedent,
         "END",
     )
 
@@ -2248,7 +2448,7 @@ class AssignmentStatementSegment(BaseSegment):
             Ref("SqlplusVariableGrammar"),
             optional=True,
         ),
-        OneOf(Sequence(Ref("ColonSegment"), Ref("EqualsSegment")), "DEFAULT"),
+        OneOf(Ref("AssignmentOperatorSegment"), "DEFAULT"),
         Ref("ExpressionSegment"),
     )
 
@@ -2263,7 +2463,9 @@ class IfExpressionStatement(BaseSegment):
 
     match_grammar = Sequence(
         Ref("IfClauseSegment"),
+        Indent,
         Ref("OneOrMoreStatementsGrammar"),
+        Dedent,
         AnyNumberOf(
             Sequence(
                 "ELSIF",
@@ -2272,12 +2474,16 @@ class IfExpressionStatement(BaseSegment):
                     Ref("TriggerPredicatesGrammar"),
                 ),
                 "THEN",
+                Indent,
                 Ref("OneOrMoreStatementsGrammar"),
+                Dedent,
             ),
         ),
         Sequence(
             "ELSE",
+            Indent,
             Ref("OneOrMoreStatementsGrammar"),
+            Dedent,
             optional=True,
         ),
         "END",
@@ -2315,7 +2521,24 @@ class CaseExpressionSegment(BaseSegment):
             "CASE",
             ImplicitIndent,
             AnyNumberOf(
-                Ref("WhenClauseSegment"),
+                Ref("WhenClauseSegment", terminators=[Ref.keyword("WHEN")]),
+                reset_terminators=True,
+                terminators=[Ref.keyword("ELSE"), Ref.keyword("END")],
+            ),
+            Ref(
+                "ElseClauseSegment",
+                optional=True,
+                reset_terminators=True,
+                terminators=[Ref.keyword("END")],
+            ),
+            Dedent,
+            "END",
+        ),
+        Sequence(
+            "CASE",
+            ImplicitIndent,
+            AnyNumberOf(
+                Ref("WhenClauseSegment", terminators=[Ref.keyword("WHEN")]),
                 reset_terminators=True,
                 terminators=[Ref.keyword("ELSE"), Ref.keyword("END")],
             ),
@@ -2338,7 +2561,7 @@ class CaseExpressionSegment(BaseSegment):
             ),
             ImplicitIndent,
             AnyNumberOf(
-                Ref("WhenClauseSegment"),
+                Ref("WhenClauseSegment", terminators=[Ref.keyword("WHEN")]),
                 reset_terminators=True,
                 terminators=[Ref.keyword("ELSE"), Ref.keyword("END")],
             ),
@@ -2529,7 +2752,9 @@ class LoopStatementSegment(BaseSegment):
     match_grammar: Matchable = Sequence(
         Ref("SingleIdentifierGrammar", optional=True),
         "LOOP",
+        Indent,
         Ref("OneOrMoreStatementsGrammar"),
+        Dedent,
         "END",
         "LOOP",
         Ref("SingleIdentifierGrammar", optional=True),
@@ -2676,7 +2901,9 @@ class BulkCollectIntoClauseSegment(BaseSegment):
         "BULK",
         "COLLECT",
         "INTO",
+        ImplicitIndent,
         Delimited(OneOf(Ref("SingleIdentifierGrammar"), Ref("SqlplusVariableGrammar"))),
+        Dedent,
     )
 
 
