@@ -3,9 +3,10 @@
 # Until we have a proper structure this will work.
 # TODO: Migrate this to the config file.
 from dataclasses import dataclass
-from typing import AbstractSet, Any, Optional
+from typing import AbstractSet, Any, Optional, Union
 
 from sqlfluff.core.config import FluffConfig
+from sqlfluff.core.helpers.string import split_comma_separated_string
 from sqlfluff.utils.reflow.depthmap import DepthInfo
 
 ConfigElementType = dict[str, str]
@@ -21,6 +22,7 @@ class BlockConfig:
     spacing_within: Optional[str] = None
     line_position: Optional[str] = None
     keyword_line_position: Optional[str] = None
+    keyword_line_position_exclusions: Union[str, list[str], None] = None
 
     def incorporate(
         self,
@@ -30,6 +32,7 @@ class BlockConfig:
         line_position: Optional[str] = None,
         config: Optional[ConfigElementType] = None,
         keyword_line_position: Optional[str] = None,
+        keyword_line_position_exclusions: Union[str, list[str], None] = None,
     ) -> None:
         """Mutate the config based on additional information."""
         config = config or {}
@@ -50,6 +53,12 @@ class BlockConfig:
             or config.get("keyword_line_position", None)
             or self.keyword_line_position
         )
+        self.keyword_line_position_exclusions = split_comma_separated_string(
+            keyword_line_position_exclusions
+            or config.get("keyword_line_position_exclusions", None)
+            or self.keyword_line_position_exclusions
+            or []
+        )
 
 
 @dataclass(frozen=True)
@@ -90,6 +99,11 @@ class ReflowConfig:
                         new_key += ":" + config_dict[seg_type]["align_within"]
                         if config_dict[seg_type].get("align_scope", None):
                             new_key += ":" + config_dict[seg_type]["align_scope"]
+                    # Optional coordinate space override
+                    if config_dict[seg_type].get("alignment_coordinate_space", None):
+                        new_key += (
+                            ":" + config_dict[seg_type]["alignment_coordinate_space"]
+                        )
                     config_dict[seg_type][key] = new_key
         return cls(_config_dict=config_dict, config_types=config_types, **kwargs)
 
